@@ -27,6 +27,7 @@
 
 #include "contacts-utils.h"
 #include "contacts-defs.h"
+#include "contacts-main.h"
 
 /* The following functions taken from
  * http://svn.o-hand.com/repos/kozo/server/src/kozo-utf8.c
@@ -156,11 +157,11 @@ kozo_utf8_strcasestrip (const char *str)
 /* Note: PHOTO and CATEGORIES are special-cased (see contacts_edit_pane_show) */
 static const ContactsField contacts_fields[] = {
 	{ "FN", E_CONTACT_FULL_NAME, NULL, FALSE, 10, TRUE },
-	{ "TEL", 0, "Phone", FALSE, 20, FALSE },
-	{ "EMAIL", 0, "Email", FALSE, 30, FALSE },
-	{ "ADR", 0, "Address", FALSE, 40, FALSE },
+	{ "TEL", 0, N_("Phone"), FALSE, 20, FALSE },
+	{ "EMAIL", 0, N_("Email"), FALSE, 30, FALSE },
+	{ "ADR", 0, N_("Address"), FALSE, 40, FALSE },
 	{ "NICKNAME", E_CONTACT_NICKNAME, NULL, FALSE, 110, TRUE },
-	{ "URL", E_CONTACT_HOMEPAGE_URL, "Homepage", FALSE, 120, FALSE },
+	{ "URL", E_CONTACT_HOMEPAGE_URL, N_("Homepage"), FALSE, 120, FALSE },
 	{ "NOTE", E_CONTACT_NOTE, NULL, TRUE, 130, TRUE },
 	{ NULL }
 };
@@ -168,25 +169,29 @@ static const ContactsField contacts_fields[] = {
 
 
 static const ContactsStructuredField contacts_sfields[] = {
-	{ "ADR", 0, "PO Box", FALSE },
-	{ "ADR", 1, "Ext.", TRUE },
-	{ "ADR", 2, "Street", TRUE },
-	{ "ADR", 3, "Locality", FALSE },
-	{ "ADR", 4, "Region", FALSE },
-	{ "ADR", 5, "Post Code", FALSE },
-	{ "ADR", 6, "Country", FALSE },
+	{ "ADR", 0, N_("PO Box"), FALSE },
+	{ "ADR", 1, N_("Ext."), TRUE },
+	{ "ADR", 2, N_("Street"), TRUE },
+	{ "ADR", 3, N_("Locality"), FALSE },
+	{ "ADR", 4, N_("Region"), FALSE },
+	{ "ADR", 5, N_("Post Code"), FALSE },
+	{ "ADR", 6, N_("Country"), FALSE },
 	{ NULL, 0, NULL, FALSE }
 };
 
-/* TODO: Would adding a struct for this be gratuititous? RB: NOT AT ALL */
-static const gchar **contacts_field_types[] = {
-	(gchar *[]){ "TEL", "Home", "Msg", "Work", "Pref", "Voice", "Fax",
+struct contacts_field_types_def {
+	char *field;
+	gchar **types;
+};
+
+static const struct contacts_field_types_def contacts_field_types[] = {
+        /* TODO: can these be i18n-ized? */
+	{ "TEL", (gchar*[]){"Home", "Msg", "Work", "Pref", "Voice", "Fax",
 			    "Cell", "Video", "Pager", "BBS", "Modem", "Car",
-			    "ISDN", "PCS", NULL },
-	(gchar *[]){ "EMAIL", "Internet", "X400", "Pref", NULL },
-	(gchar *[]){ "ADR", "Dom", "Intl", "Postal", "Parcel", "Home", "Work",
-			    "Pref", NULL },
-	(gchar *[]){ NULL }
+			    "ISDN", "PCS", NULL }},
+	{ "EMAIL", (gchar*[]){"Internet", "X400", "Pref", NULL}},
+	{ "ADR", (gchar*[]){"Dom", "Intl", "Postal", "Parcel", "Home", "Work", "Pref", NULL}},
+	{ NULL, NULL }
 };
 
 const gchar **
@@ -194,9 +199,9 @@ contacts_get_field_types (const gchar *attr_name)
 {
 	guint i;
 
-	for (i = 0; contacts_field_types[i][0]; i++) {
-		if (strcmp (contacts_field_types[i][0], attr_name) == 0)
-			return (const gchar **)contacts_field_types[i];
+	for (i = 0; contacts_field_types[i].field; i++) {
+		if (strcmp (contacts_field_types[i].field, attr_name) == 0)
+			return (const gchar **)contacts_field_types[i].types;
 	}
 	
 	return NULL;
@@ -253,7 +258,7 @@ const gchar *
 contacts_field_pretty_name (const ContactsField *field)
 {
 	if (field->pretty_name) {
-		return field->pretty_name;
+		return gettext(field->pretty_name);
 	} else if (field->econtact_field > 0) {
 		return e_contact_pretty_name (field->econtact_field);
 	} else
@@ -261,16 +266,10 @@ contacts_field_pretty_name (const ContactsField *field)
 }
 
 EContact *
-contacts_contact_from_selection (GtkTreeSelection *selection,
-				 GHashTable *contacts_table)
+contacts_contact_from_tree_path (GtkTreeModel *model, GtkTreePath *path, GHashTable *contacts_table)
 {
-	GtkTreeModel *model;
 	GtkTreeIter iter;
-	
-	if (!selection || !GTK_IS_TREE_SELECTION (selection))
-		return NULL;
-		
-	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+	if (gtk_tree_model_get_iter (model, &iter, path)) {
 		const gchar *uid;
 		EContactListHash *hash;
 		gtk_tree_model_get (model, &iter, CONTACT_UID_COL, &uid, -1);
@@ -283,6 +282,30 @@ contacts_contact_from_selection (GtkTreeSelection *selection,
 	return NULL;
 }
 
+EContact *
+contacts_contact_from_selection (GtkTreeSelection *selection,
+				 GHashTable *contacts_table)
+{
+	GtkTreeModel *model;
+	GList *selected_paths;
+	GtkTreePath *path;
+	EContact *result;
+	
+	if (!selection || !GTK_IS_TREE_SELECTION (selection))
+		return NULL;
+
+	selected_paths = gtk_tree_selection_get_selected_rows (selection, &model);
+	if (!selected_paths)
+		return NULL;
+	path = selected_paths->data;
+
+	result = contacts_contact_from_tree_path (model, path, contacts_table);
+
+	g_list_foreach (selected_paths, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (selected_paths);
+	return result;
+}
+
 EContact *
 contacts_get_selected_contact (GladeXML *xml, GHashTable *contacts_table)
 {
@@ -302,6 +325,35 @@ contacts_get_selected_contact (GladeXML *xml, GHashTable *contacts_table)
 	return contact;
 }
 
+void
+contacts_set_selected_contact (GladeXML *xml, const gchar *uid)
+{
+	GtkTreeView *treeview;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+
+	treeview = GTK_TREE_VIEW (
+		glade_xml_get_widget (xml, "contacts_treeview"));
+	model = /*gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (*/
+		gtk_tree_view_get_model (treeview)/*))*/;
+	
+	if (!gtk_tree_model_get_iter_first (model, &iter)) return;
+	
+	do {
+		const gchar *ruid;
+		gtk_tree_model_get (model, &iter, CONTACT_UID_COL, &ruid, -1);
+		if (strcmp (uid, ruid) == 0) {
+			GtkTreeSelection *selection =
+				gtk_tree_view_get_selection (treeview);
+			if (selection)
+				gtk_tree_selection_select_iter (
+					selection, &iter);
+			return;
+		}
+	} while (gtk_tree_model_iter_next (model, &iter));
+
+}
+
 /* Helper method to set edit/delete sensitive/insensitive */
 void
 contacts_set_available_options (GladeXML *xml, gboolean new, gboolean open,
@@ -357,8 +409,8 @@ contacts_load_photo (EContact *contact)
 			switch (photo->type) {
 			case E_CONTACT_PHOTO_TYPE_INLINED :
 				gdk_pixbuf_loader_write (loader,
-					photo->inlined.data,
-					photo->inlined.length, NULL);
+					photo->data.inlined.data,
+					photo->data.inlined.length, NULL);
 				break;
 			case E_CONTACT_PHOTO_TYPE_URI :
 			default :
@@ -505,7 +557,7 @@ contacts_choose_photo (GtkWidget *button, EContact *contact)
 	/* Note: I don't use the GTK_WINDOW cast as gtk_widget_get_ancestor
 	 * can return NULL and this would probably throw a critical Gtk error.
 	 */
-	filechooser = gtk_file_chooser_dialog_new ("Open image",
+	filechooser = gtk_file_chooser_dialog_new (_("Open image"),
 						   (GtkWindow *)
 						   gtk_widget_get_ancestor (
 						   	button,
@@ -515,7 +567,7 @@ contacts_choose_photo (GtkWidget *button, EContact *contact)
 						   GTK_RESPONSE_CANCEL,
 						   GTK_STOCK_OPEN,
 						   GTK_RESPONSE_ACCEPT,
-						   "No image",
+						   _("No image"),
 						   NO_IMAGE,
 						   NULL);
 	/* Set filter by supported EContactPhoto image types */
@@ -541,19 +593,19 @@ contacts_choose_photo (GtkWidget *button, EContact *contact)
 		if (filename) {
 			if (contact) {
 				EContactPhoto new_photo;
-				char **data;
+				guchar **data;
 				int *length;
 #if HAVE_PHOTO_TYPE
 				new_photo.type = E_CONTACT_PHOTO_TYPE_INLINED;
-				data = &new_photo.inlined.data;
-				length = &new_photo.inlined.length;
-				new_photo.inlined.mime_type = NULL;
+				data = &new_photo.data.inlined.data;
+				length = &new_photo.data.inlined.length;
+				new_photo.data.inlined.mime_type = NULL;
 #else
 				data = &new_photo.data;
 				length = &new_photo.length;
 #endif
 				if (g_file_get_contents (filename, 
-							 data,
+							 (gchar **)data,
 							 (gsize *)length,
 							 NULL)) {
 					e_contact_set (contact, E_CONTACT_PHOTO,
@@ -786,7 +838,7 @@ contacts_set_widget_desensitive_recurse (GtkWidget *widget, GList **widgets)
 		}
 		
 		if (GTK_IS_TABLE (widget) || GTK_IS_HBOX (widget) ||
-		    GTK_IS_VBOX (widget)) {
+		    GTK_IS_VBOX (widget) || GTK_IS_EVENT_BOX (widget)) {
 			GList *c, *children = gtk_container_get_children (
 				GTK_CONTAINER (widget));
 			
